from pydantic_settings import BaseSettings, SettingsConfigDict

from src.core.common.constants import PydanticEnvPrefixEnum, EnvFileLocation, EnvironmentEnum
from src.core.settings.account import AccountSettings
from src.core.settings.bot import BotSettings

from src.core.settings.database import DatabaseSettings
from src.core.settings.gemini import GeminiSettings
from src.core.settings.groq import GroqSettings
from src.core.settings.notify import NotifySettings
from src.core.settings.rabbitmq import RabbitmqSettings
from src.core.settings.redis import RedisSettings

class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        case_sensitive=True,
        env_nested_delimiter='__',
        env_prefix=PydanticEnvPrefixEnum.APP.value,
        env_file=(
            EnvFileLocation.PRODUCTION,
            # EnvFileLocation.TESTING,
        )
    )

    MODE: EnvironmentEnum

    POSTGRES: DatabaseSettings

    NOTIFY: NotifySettings = NotifySettings()
    BOT: BotSettings
    ACCOUNT: AccountSettings

    GROQ: GroqSettings
    GEMINI: GeminiSettings
    REDIS: RedisSettings
    RABBIT: RabbitmqSettings


settings = Settings() # type:ignore